@@ -153,49 +153,52 @@ litest_fail_comparison_ptr(const char *file,
 
 enum litest_device_type {
 	LITEST_NO_DEVICE = -1,
-	LITEST_SYNAPTICS_CLICKPAD = -2,
-	LITEST_SYNAPTICS_TOUCHPAD = -3,
-	LITEST_SYNAPTICS_TOPBUTTONPAD = -4,
-	LITEST_BCM5974 = -5,
-	LITEST_KEYBOARD = -6,
-	LITEST_TRACKPOINT = -7,
-	LITEST_MOUSE = -8,
-	LITEST_WACOM_TOUCH = -9,
-	LITEST_ALPS_SEMI_MT = -10,
-	LITEST_GENERIC_SINGLETOUCH = -11,
-	LITEST_MS_SURFACE_COVER = -12,
-	LITEST_QEMU_TABLET = -13,
-	LITEST_XEN_VIRTUAL_POINTER = -14,
-	LITEST_VMWARE_VIRTMOUSE = -15,
-	LITEST_SYNAPTICS_HOVER_SEMI_MT = -16,
-	LITEST_SYNAPTICS_TRACKPOINT_BUTTONS = -17,
-	LITEST_PROTOCOL_A_SCREEN = -18,
-	LITEST_WACOM_FINGER = -19,
-	LITEST_KEYBOARD_BLACKWIDOW = -20,
-	LITEST_WHEEL_ONLY = -21,
-	LITEST_MOUSE_ROCCAT = -22,
-	LITEST_LOGITECH_TRACKBALL = -23,
-	LITEST_ATMEL_HOVER = -24,
-	LITEST_ALPS_DUALPOINT = -25,
-	LITEST_MOUSE_LOW_DPI = -26,
-	LITEST_GENERIC_MULTITOUCH_SCREEN = -27,
-	LITEST_NEXUS4_TOUCH_SCREEN = -28,
-	LITEST_MAGIC_TRACKPAD = -29,
-	LITEST_ELANTECH_TOUCHPAD = -30,
-	LITEST_MOUSE_GLADIUS = -31,
-	LITEST_MOUSE_WHEEL_CLICK_ANGLE = -32,
-	LITEST_APPLE_KEYBOARD = -33,
-	LITEST_ANKER_MOUSE_KBD = -34,
-	LITEST_WACOM_BAMBOO = -35,
-	LITEST_WACOM_CINTIQ = -36,
-	LITEST_WACOM_INTUOS = -37,
-	LITEST_WACOM_ISDV4 = -38,
-	LITEST_WALTOP = -39,
-	LITEST_HUION_TABLET = -40,
-	LITEST_CYBORG_RAT = -41,
-	LITEST_YUBIKEY = -42,
-	LITEST_SYNAPTICS_I2C = -43,
-	LITEST_WACOM_CINTIQ_24HD = -44,
+	LITEST_SYNAPTICS_CLICKPAD = -1000,
+	LITEST_SYNAPTICS_TOUCHPAD,
+	LITEST_SYNAPTICS_TOPBUTTONPAD,
+	LITEST_BCM5974,
+	LITEST_KEYBOARD,
+	LITEST_TRACKPOINT,
+	LITEST_MOUSE,
+	LITEST_WACOM_TOUCH,
+	LITEST_ALPS_SEMI_MT,
+	LITEST_GENERIC_SINGLETOUCH,
+	LITEST_MS_SURFACE_COVER,
+	LITEST_QEMU_TABLET,
+	LITEST_XEN_VIRTUAL_POINTER,
+	LITEST_VMWARE_VIRTMOUSE,
+	LITEST_SYNAPTICS_HOVER_SEMI_MT,
+	LITEST_SYNAPTICS_TRACKPOINT_BUTTONS,
+	LITEST_PROTOCOL_A_SCREEN,
+	LITEST_WACOM_FINGER,
+	LITEST_KEYBOARD_BLACKWIDOW,
+	LITEST_WHEEL_ONLY,
+	LITEST_MOUSE_ROCCAT,
+	LITEST_LOGITECH_TRACKBALL,
+	LITEST_ATMEL_HOVER,
+	LITEST_ALPS_DUALPOINT,
+	LITEST_MOUSE_LOW_DPI,
+	LITEST_GENERIC_MULTITOUCH_SCREEN,
+	LITEST_NEXUS4_TOUCH_SCREEN,
+	LITEST_MAGIC_TRACKPAD,
+	LITEST_ELANTECH_TOUCHPAD,
+	LITEST_MOUSE_GLADIUS,
+	LITEST_MOUSE_WHEEL_CLICK_ANGLE,
+	LITEST_APPLE_KEYBOARD,
+	LITEST_ANKER_MOUSE_KBD,
+	LITEST_WACOM_BAMBOO,
+	LITEST_WACOM_CINTIQ,
+	LITEST_WACOM_INTUOS,
+	LITEST_WACOM_ISDV4,
+	LITEST_WALTOP,
+	LITEST_HUION_TABLET,
+	LITEST_CYBORG_RAT,
+	LITEST_YUBIKEY,
+	LITEST_SYNAPTICS_I2C,
+	LITEST_WACOM_CINTIQ_24HD,
+	LITEST_MULTITOUCH_FUZZ_SCREEN,
+	LITEST_WACOM_INTUOS3_PAD,
+	LITEST_WACOM_INTUOS5_PAD,
 };
 
 enum litest_device_feature {
@@ -222,6 +225,9 @@ enum litest_device_feature {
 	LITEST_DISTANCE = 1 << 18,
 	LITEST_TOOL_SERIAL = 1 << 19,
 	LITEST_TILT = 1 << 20,
+	LITEST_TABLET_PAD = 1 << 21,
+	LITEST_RING = 1 << 22,
+	LITEST_STRIP = 1 << 23,
 };
 
 struct litest_device {
@@ -403,6 +409,13 @@ litest_touch_move_to(struct litest_device *d,
 		     double x_from, double y_from,
 		     double x_to, double y_to,
 		     int steps, int sleep_ms);
+void
+litest_touch_move_to_extended(struct litest_device *d,
+			      unsigned int slot,
+			      double x_from, double y_from,
+			      double x_to, double y_to,
+			      struct axis_replacement *axes,
+			      int steps, int sleep_ms);
 
 void
 litest_touch_move_two_touches(struct litest_device *d,
@@ -432,6 +445,24 @@ litest_tablet_motion(struct litest_device *d,
 		     int x, int y,
 		     struct axis_replacement *axes);
 
+void
+litest_pad_ring_start(struct litest_device *d, double value);
+
+void
+litest_pad_ring_change(struct litest_device *d, double value);
+
+void
+litest_pad_ring_end(struct litest_device *d);
+
+void
+litest_pad_strip_start(struct litest_device *d, double value);
+
+void
+litest_pad_strip_change(struct litest_device *d, double value);
+
+void
+litest_pad_strip_end(struct litest_device *d);
+
 void
 litest_hover_start(struct litest_device *d,
 		   unsigned int slot,
@@ -518,6 +549,19 @@ struct libinput_event_tablet_tool *
 litest_is_tablet_event(struct libinput_event *event,
 		       enum libinput_event_type type);
 
+struct libinput_event_tablet_pad *
+litest_is_pad_button_event(struct libinput_event *event,
+			   unsigned int button,
+			   enum libinput_button_state state);
+struct libinput_event_tablet_pad *
+litest_is_pad_ring_event(struct libinput_event *event,
+			 unsigned int number,
+			 enum libinput_tablet_pad_ring_axis_source source);
+struct libinput_event_tablet_pad *
+litest_is_pad_strip_event(struct libinput_event *event,
+			  unsigned int number,
+			  enum libinput_tablet_pad_strip_axis_source source);
+
 void
 litest_assert_button_event(struct libinput *li,
 			   unsigned int button,
@@ -541,6 +585,10 @@ void
 litest_assert_tablet_proximity_event(struct libinput *li,
 				     enum libinput_tablet_tool_proximity_state state);
 
+void
+litest_assert_pad_button_event(struct libinput *li,
+				    unsigned int button,
+				    enum libinput_button_state state);
 struct libevdev_uinput *
 litest_create_uinput_device(const char *name,
 			    struct input_id *id,
